from django.db.models.functions import Lower
from django.urls import reverse

from django.db import models
from django.db.models import F, Q, CheckConstraint
from django.core.exceptions import ValidationError

from ipam.fields import IPNetworkField
from netbox.models import NetBoxModel
from netbox_routing.choices.objects import PermitDenyChoices


__all__ = (
    'RouteMap',
    'RouteMapEntry',
    'PrefixList',
    'PrefixListEntry'
)


class PermitDenyChoiceMixin:
    def get_type_color(self):
        return PermitDenyChoices.colors.get(self.type)


class RouteMap(NetBoxModel):
    name = models.CharField(
        max_length=255
    )

    clone_fields = ()
    prerequisite_models = ()

    class Meta:
        ordering = ['name', ]
        constraints = (
            models.UniqueConstraint(
                Lower('name'),
                name='%(app_label)s_%(class)s_unique_name',
                violation_error_message="Name must be unique."
            ),
        )

    def __str__(self):
        return f'{self.name}'

    def get_absolute_url(self):
        return reverse('plugins:netbox_routing:routemap', args=[self.pk])

class RouteMapEntry(PermitDenyChoiceMixin, NetBoxModel):
    route_map = models.ForeignKey(
        to="netbox_routing.RouteMap",
        on_delete=models.PROTECT,
        related_name='entries',
        verbose_name='Route Map'
    )
    type = models.CharField(max_length=6, choices=PermitDenyChoices)
    sequence = models.PositiveSmallIntegerField()

    clone_fields = (
        'route_map', 'type',
    )
    prerequisite_models = (
        'netbox_routing.RouteMap',
    )

    class Meta:
        ordering = ['route_map', 'sequence']
        constraints = (
            models.UniqueConstraint(
                'route_map', 'sequence',
                name='%(app_label)s_%(class)s_unique_routemap_sequence',
                violation_error_message="Route Map sequence must be unique."
            ),
        )

    def __str__(self):
        return f'{self.route_map.name} {self.sequence}'

    def get_absolute_url(self):
        return reverse('plugins:netbox_routing:routemapentry', args=[self.pk])


class PrefixList(NetBoxModel):
    name = models.CharField(
        max_length=255
    )

    clone_fields = ()
    prerequisite_models = ()

    class Meta:
        ordering = ['name', ]
        constraints = (
            models.UniqueConstraint(
                Lower('name'),
                name='%(app_label)s_%(class)s_unique_name',
                violation_error_message="Name must be unique."
            ),
        )

    def __str__(self):
        return f'{self.name}'

    def get_absolute_url(self):
        return reverse('plugins:netbox_routing:prefixlist', args=[self.pk])


class PrefixListEntry(PermitDenyChoiceMixin, NetBoxModel):
    prefix_list = models.ForeignKey(
        to="netbox_routing.PrefixList",
        on_delete=models.PROTECT,
        related_name='entries',
        verbose_name='Prefix List'
    )
    sequence = models.PositiveSmallIntegerField()
    type = models.CharField(max_length=6, choices=PermitDenyChoices)
    prefix = IPNetworkField(help_text='IPv4 or IPv6 network with mask')
    ge = models.PositiveSmallIntegerField(
        verbose_name='GE',
        null=True,
        blank=True,
    )
    le = models.PositiveSmallIntegerField(
        verbose_name='LE',
        null=True,
        blank=True,
    )

    clone_fields = (
        'prefix_list', 'type',
    )
    prerequisite_models = (
        'netbox_routing.PrefixList',
    )

    class Meta:
        ordering = ['prefix_list', 'sequence']
        constraints = (
            models.UniqueConstraint(
                'prefix_list', 'sequence',
                name='%(app_label)s_%(class)s_unique_prefixlist_sequence',
                violation_error_message="Prefix List sequence must be unique."
            ),
        )

    def __str__(self):
        return f'{self.prefix_list.name} {self.sequence}'

    def get_absolute_url(self):
        return reverse('plugins:netbox_routing:prefixlistentry', args=[self.pk])

    def clean(self):
        super().clean()

        if self.prefix.version == 6:
            if self.le is not None and self.le > 128:
                raise ValidationError({
                    'le': 'LE value cannot be longer then 128'
                })
            if self.ge is not None and self.ge > 128:
                raise ValidationError({
                    'ge': 'GE value cannot be longer then 128'
                })
        elif self.prefix.version == 4:
            if self.le is not None and self.le > 32:
                raise ValidationError({
                    'le': 'LE value cannot be longer then 32'
                })
            if self.ge is not None and self.ge > 32:
                raise ValidationError({
                    'ge': 'GE value cannot be longer then 32'
                })

        if self.ge and self.le and self.ge < self.le:
            raise ValidationError({
                    'ge': 'GE cannot be more then LE',
                    'le': 'LE cannot be less then GE'
            })

        if self.ge is not None and self.prefix.prefix.prefixlen >= self.ge:
            raise ValidationError('Prefix\'s length cannot be longer then greater or equals value')

        if self.le is not None and self.prefix.prefix.prefixlen >= self.le:
            raise ValidationError('Prefix\'s length cannot be longer then greater or equals value')




from django import forms
from django.utils.translation import gettext as _

from dcim.models import Interface, Device
from netbox.forms import NetBoxModelFilterSetForm
from netbox_routing.choices import AuthenticationChoices
from utilities.forms import BOOLEAN_WITH_BLANK_CHOICES, add_blank_choice
from utilities.forms.fields import TagFilterField, DynamicModelMultipleChoiceField

from netbox_routing.models import OSPFInstance, OSPFArea, OSPFInterface


__all__ = (
    'OSPFAreaFilterForm',
    'OSPFInstanceFilterForm',
    'OSPFInterfaceFilterForm',
)


class OSPFInstanceFilterForm(NetBoxModelFilterSetForm):
    device = DynamicModelMultipleChoiceField(
        queryset=Device.objects.all(),
        required=False,
        selector=True,
        label=_('Device'),
    )
    model = OSPFInstance
    fieldsets = (
        (None, ('q', 'filter_id', 'tag', 'device')),
    )
    tag = TagFilterField(model)


class OSPFAreaFilterForm(NetBoxModelFilterSetForm):
    model = OSPFArea
    fieldsets = (
        (None, ('q', 'filter_id', 'tag', )),
    )
    tag = TagFilterField(model)


class OSPFInterfaceFilterForm(NetBoxModelFilterSetForm):
    model = OSPFInterface
    fieldsets = (
        (None, ('q', 'filter_id', 'tag', )),
        ('OSPF', ('instance', 'area')),
        ('Device', ('interface', )),
        ('Attributes', ('priority', 'bfd', 'authentication')),
    )
    instance = DynamicModelMultipleChoiceField(
        queryset=OSPFInstance.objects.all(),
        required=False,
        selector=True,
        label=_('Instance'),
    )
    area = DynamicModelMultipleChoiceField(
        queryset=OSPFArea.objects.all(),
        required=False,
        selector=True,
        label=_('Area'),
    )
    interface = DynamicModelMultipleChoiceField(
        queryset=Interface.objects.all(),
        required=False,
        selector=True,
        label=_('Interface'),
    )
    bfd = forms.NullBooleanField(
        required=False,
        label='BFD Enabled',
        widget=forms.Select(
            choices=BOOLEAN_WITH_BLANK_CHOICES
        )
    )
    priority = forms.IntegerField(
        required=False
    )
    authentication = forms.ChoiceField(
        choices=add_blank_choice(AuthenticationChoices),
        required=False
    )
    tag = TagFilterField(model)




from netbox.views.generic import ObjectListView, ObjectView, ObjectEditView, ObjectDeleteView, ObjectChildrenView, \
    BulkDeleteView, BulkEditView
from netbox_routing.filtersets import PrefixListFilterSet, PrefixListEntryFilterSet, RouteMapEntryFilterSet, \
    RouteMapFilterSet
from netbox_routing.forms import PrefixListFilterForm, PrefixListForm, PrefixListEntryFilterForm, \
    PrefixListEntryForm, RouteMapEntryForm, RouteMapEntryFilterForm, RouteMapForm, RouteMapFilterForm, \
    PrefixListEntryBulkEditForm, RouteMapEntryBulkEditForm
from netbox_routing.models import PrefixList, PrefixListEntry, RouteMapEntry, RouteMap
from netbox_routing.tables.objects import PrefixListTable, PrefixListEntryTable, RouteMapEntryTable, RouteMapTable
from utilities.views import register_model_view, ViewTab


#
# Prefix List
#
@register_model_view(PrefixList, name='list')
class PrefixListListView(ObjectListView):
    queryset = PrefixList.objects.all()
    table = PrefixListTable
    filterset = PrefixListFilterSet
    filterset_form = PrefixListFilterForm


@register_model_view(PrefixList)
class PrefixListView(ObjectView):
    queryset = PrefixList.objects.all()
    template_name = 'netbox_routing/prefixlist.html'


@register_model_view(PrefixList, name='entries')
class PrefixListEntriesView(ObjectChildrenView):
    template_name = 'netbox_routing/objecttable.html'
    queryset = PrefixList.objects.all()
    child_model = PrefixListEntry
    table = PrefixListEntryTable
    filterset = PrefixListEntryFilterSet
    actions = ['add', 'edit', 'delete', 'bulk_edit', 'bulk_delete']
    tab = ViewTab(
        label='Entries',
        badge=lambda obj: PrefixListEntry.objects.filter(prefix_list=obj).count(),
    )

    def get_children(self, request, parent):
        return self.child_model.objects.filter(prefix_list=parent)

    def get_extra_context(self, request, instance):

        return {
            'url_parent': 'plugins:netbox_routing:prefixlist_entries',
            'url_add': 'plugins:netbox_routing:prefixlistentry_add',
            'url_bulk_edit': 'plugins:netbox_routing:prefixlistentry_bulk_edit',
            'url_bulk_delete': 'plugins:netbox_routing:prefixlistentry_bulk_delete',
            'parent_name': 'prefix_list',
            'parent_view': 'prefixlist',
            'parent_view_path': 'entries',
            'view': 'prefixlistentry'
        }


class PrefixListEditView(ObjectEditView):
    queryset = PrefixList.objects.all()
    form = PrefixListForm


class PrefixListDeleteView(ObjectDeleteView):
    pass


#
# Prefix List Entry
#
class PrefixListEntryListView(ObjectListView):
    queryset = PrefixListEntry.objects.all()
    table = PrefixListEntryTable
    filterset = PrefixListEntryFilterSet
    filterset_form = PrefixListEntryFilterForm


class PrefixListEntryView(ObjectView):
    queryset = PrefixListEntry.objects.all()
    template_name = 'netbox_routing/prefixlistentry.html'


class PrefixListEntryEditView(ObjectEditView):
    queryset = PrefixListEntry.objects.all()
    form = PrefixListEntryForm


class PrefixListEntryDeleteView(ObjectDeleteView):
    pass


class PrefixListEntryBulkEditView(BulkEditView):
    queryset = PrefixListEntry.objects.all()
    filterset = PrefixListEntryFilterSet
    table = PrefixListEntryTable
    form = PrefixListEntryBulkEditForm


class PrefixListEntryBulkDeleteView(BulkDeleteView):
    queryset = PrefixListEntry.objects.all()
    filterset = PrefixListEntryFilterSet
    table = PrefixListEntryTable


#
# Route Map
#
class RouteMapListView(ObjectListView):
    queryset = RouteMap.objects.all()
    table = RouteMapTable
    filterset = RouteMapFilterSet
    filterset_form = RouteMapFilterForm


class RouteMapView(ObjectView):
    queryset = RouteMap.objects.all()
    template_name = 'netbox_routing/routemap.html'


@register_model_view(RouteMap, name='entries')
class RouteMapEntriesView(ObjectChildrenView):
    template_name = 'netbox_routing/objecttable.html'
    queryset = RouteMap.objects.all()
    child_model = RouteMapEntry
    table = RouteMapEntryTable
    filterset = RouteMapEntryFilterSet
    actions = ['add', 'edit', 'delete', 'bulk_edit', 'bulk_delete']
    tab = ViewTab(
        label='Entries',
        badge=lambda obj: RouteMapEntry.objects.filter(route_map=obj).count(),
    )

    def get_children(self, request, parent):
        return self.child_model.objects.filter(route_map=parent)

    def get_extra_context(self, request, instance):

        return {
            'url_parent': 'plugins:netbox_routing:routemap_entries',
            'url_add': 'plugins:netbox_routing:routemapentry_add',
            'url_bulk_edit': 'plugins:netbox_routing:routemapentry_bulk_edit',
            'url_bulk_delete': 'plugins:netbox_routing:routemapentry_bulk_delete',
            'parent_name': 'route_map',
            'parent_view': 'routemap',
            'parent_view_path': 'entries',
            'view': 'routemapentry'
        }


class RouteMapEditView(ObjectEditView):
    queryset = RouteMap.objects.all()
    form = RouteMapForm


class RouteMapDeleteView(ObjectDeleteView):
    pass


#
# Route Map Entry
#
class RouteMapEntryListView(ObjectListView):
    queryset = RouteMapEntry.objects.all()
    table = RouteMapEntryTable
    filterset = RouteMapEntryFilterSet
    filterset_form = RouteMapEntryFilterForm


class RouteMapEntryView(ObjectView):
    queryset = RouteMapEntry.objects.all()
    template_name = 'netbox_routing/routemapentry.html'


class RouteMapEntryEditView(ObjectEditView):
    queryset = RouteMapEntry.objects.all()
    form = RouteMapEntryForm


class RouteMapEntryDeleteView(ObjectDeleteView):
    pass


class RouteMapEntryBulkEditView(BulkEditView):
    queryset = RouteMapEntry.objects.all()
    filterset = RouteMapEntryFilterSet
    table = RouteMapEntryTable
    form = RouteMapEntryBulkEditForm


class RouteMapEntryBulkDeleteView(BulkDeleteView):
    queryset = RouteMapEntry.objects.all()
    filterset = RouteMapEntryFilterSet
    table = RouteMapEntryTable



from ipam.models import VRF
from netbox.forms import NetBoxModelFilterSetForm
from netbox_routing.models import StaticRoute
from utilities.forms.fields import DynamicModelMultipleChoiceField, TagFilterField
from django.utils.translation import gettext as _


class StaticRouteFilterForm(NetBoxModelFilterSetForm):
    model = StaticRoute
    fieldsets = (
        (None, ('q', 'filter_id', 'tag', 'vrf')),
    )
    vrf = DynamicModelMultipleChoiceField(
        queryset=VRF.objects.all(),
        required=False,
        selector=True,
        label=_('VRF'),
    )
    tag = TagFilterField(model)


from django.db import models
from django.db.models import CheckConstraint, Q
from django.urls import reverse

from ipam.fields import IPNetworkField
from netbox.models import NetBoxModel
from netbox_routing.fields.ip import IPAddressField


__all__ = (
    'StaticRoute'
)


class StaticRoute(NetBoxModel):
    devices = models.ManyToManyField(
        to='dcim.Device',
        related_name='static_routes'
    )
    vrf = models.ForeignKey(
        to='ipam.VRF',
        on_delete=models.PROTECT,
        related_name='staticroutes',
        blank=True,
        null=True,
        verbose_name='VRF'
    )
    prefix = IPNetworkField(help_text='IPv4 or IPv6 network with mask')
    next_hop = IPAddressField()
    name = models.CharField(
        max_length=50,
        verbose_name='Name',
        blank=True,
        null=True,
        help_text='Optional name for this static route'
    )
    metric = models.PositiveSmallIntegerField(
        verbose_name='Metric'
    )
    permanent = models.BooleanField()

    clone_fields = (
        'vrf', 'metric', 'permanent'
    )
    prerequisite_models = (
        'dcim.Device',
        'ipam.VRF',
    )

    class Meta:
        ordering = ['vrf', 'prefix', 'metric']
        constraints = (
            CheckConstraint(check=Q(Q(metric__lte=255) & Q(metric__gte=0)), name='metric_gte_lte'),
            models.UniqueConstraint(
                'vrf', 'prefix', 'next_hop',
                name='%(app_label)s_%(class)s_unique_vrf_prefix_nexthop',
                violation_error_message="VRF, Prefix and Next Hop must be unique."
            ),
        )

    def __str__(self):
        if self.vrf is None:
            return f'{self.prefix} NH {self.next_hop}'
        return f'{self.prefix} VRF {self.vrf} NH {self.next_hop}'

    def get_absolute_url(self):
        return reverse('plugins:netbox_routing:staticroute', args=[self.pk])



from rest_framework import serializers

from dcim.api.nested_serializers import NestedDeviceSerializer
from ipam.api.nested_serializers import NestedVRFSerializer
from netbox.api.serializers import NetBoxModelSerializer
from netbox_routing.models import StaticRoute


__all__ = (
    'StaticRouteSerializer'
)


class StaticRouteSerializer(NetBoxModelSerializer):
    url = serializers.HyperlinkedIdentityField(view_name='plugins-api:netbox_routing-api:staticroute-detail')
    devices = NestedDeviceSerializer(many=True)
    vrf = NestedVRFSerializer()

    class Meta:
        model = StaticRoute
        fields = ('url', 'id', 'display', 'devices', 'vrf', 'prefix', 'next_hop', 'name', 'metric', 'permanent')



from django.utils.translation import gettext as _

from dcim.models import Interface
from netbox.forms import NetBoxModelImportForm
from utilities.forms.fields import CSVModelChoiceField

from netbox_routing.models import OSPFInstance, OSPFArea, OSPFInterface


__all__ = (
    'OSPFInstanceImportForm',
    'OSPFAreaImportForm',
    'OSPFInterfaceImportForm',
)


class OSPFInstanceImportForm(NetBoxModelImportForm):
    device = CSVModelChoiceField(
        queryset=Interface.objects.all(),
        required=False,
        to_field_name='name',
        help_text=_('Name of device')
    )

    class Meta:
        model = OSPFInstance
        fields = ('name', 'router_id', 'process_id', 'device', 'tags')


class OSPFAreaImportForm(NetBoxModelImportForm):

    class Meta:
        model = OSPFArea
        fields = ('area_id', 'tags')


class OSPFInterfaceImportForm(NetBoxModelImportForm):
    instance = CSVModelChoiceField(
        queryset=OSPFInstance.objects.all(),
        required=False,
        to_field_name='name',
        help_text=_('Name of OSPF Instance')
    )
    area = CSVModelChoiceField(
        queryset=OSPFArea.objects.all(),
        required=False,
        to_field_name='name',
        help_text=_('Area ID')
    )
    interface = CSVModelChoiceField(
        queryset=Interface.objects.all(),
        required=False,
        to_field_name='name',
        help_text=_('Name of interface')
    )

    class Meta:
        model = OSPFInterface
        fields = ('instance', 'area', 'interface', 'tags')



import django_filters
import netaddr
from django.db.models import Q

from netbox.filtersets import NetBoxModelFilterSet
from netbox_routing.models import StaticRoute


class StaticRouteFilterSet(NetBoxModelFilterSet):

    prefix = django_filters.CharFilter(
        method='filter_prefix',
        label='Prefix',
    )

    next_hop = django_filters.CharFilter(
        method='filter_address',
        label='Prefix',
    )

    class Meta:
        model = StaticRoute
        fields = ('vrf', 'prefix', 'devices', 'metric', 'next_hop')

    def search(self, queryset, name, value):
        if not value.strip():
            return queryset
        qs_filter = (
            Q(devices__name__icontains=value) |
            Q(vrf__name__icontains=value) |
            Q(vrf__rd__icontains=value) |
            Q(prefix__icontains=value) |
            Q(next_hop__icontains=value) |
            Q(name__icontains=value)
        )
        return queryset.filter(qs_filter).distinct()

    def filter_prefix(self, queryset, name, value):
        if not value.strip():
            return queryset
        try:
            query = str(netaddr.IPNetwork(value).cidr)
            return queryset.filter(prefix=query)
        except (netaddr.AddrFormatError, ValueError):
            return queryset.none()

    def filter_address(self, queryset, name, value):
        if not value.strip():
            return queryset
        try:
            query = str(netaddr.IPAddress(value))
            return queryset.filter(prefix=query)
        except (netaddr.AddrFormatError, ValueError):
            return queryset.none()



from rest_framework import serializers

from dcim.api.nested_serializers import NestedInterfaceSerializer
from netbox.api.serializers import WritableNestedSerializer
from netbox_routing.models import OSPFInstance, OSPFArea, OSPFInterface


__all__ = (
    'NestedOSPFInstanceSerializer',
    'NestedOSPFAreaSerializer',
    'NestedOSPFInterfaceSerializer',
)


class NestedOSPFInstanceSerializer(WritableNestedSerializer):
    url = serializers.HyperlinkedIdentityField(view_name='plugins-api:netbox_routing-api:ospfinstance-detail')

    class Meta:
        model = OSPFInstance
        fields = ('url', 'id', 'display', 'name', 'router_id', 'process_id')


class NestedOSPFAreaSerializer(WritableNestedSerializer):
    url = serializers.HyperlinkedIdentityField(view_name='plugins-api:netbox_routing-api:ospfarea-detail')

    class Meta:
        model = OSPFArea
        fields = ('url', 'id', 'display', 'area_id')


class NestedOSPFInterfaceSerializer(WritableNestedSerializer):
    url = serializers.HyperlinkedIdentityField(view_name='plugins-api:netbox_routing-api:ospfarea-detail')
    instance = NestedOSPFInstanceSerializer()
    area = NestedOSPFAreaSerializer()
    interface = NestedInterfaceSerializer()

    class Meta:
        model = OSPFInterface
        fields = ('url', 'id', 'display', 'instance', 'area', 'interface')




from django.utils.translation import gettext as _

from netbox.forms import NetBoxModelBulkEditForm
from netbox_routing.models import PrefixList, PrefixListEntry, RouteMapEntry, RouteMap
from utilities.forms.fields import DynamicModelChoiceField


__all__ = (
    'PrefixListEntryBulkEditForm',
    'RouteMapEntryBulkEditForm'
)


class PrefixListEntryBulkEditForm(NetBoxModelBulkEditForm):
    prefix_list = DynamicModelChoiceField(
        queryset=PrefixList.objects.all(),
        label=_('Prefix List'),
        required=False,
        selector=True
    )

    model = PrefixListEntry
    fieldsets = (
        (None, ('prefix_list', )),
    )
    nullable_fields = ()


class RouteMapEntryBulkEditForm(NetBoxModelBulkEditForm):
    route_map = DynamicModelChoiceField(
        queryset=RouteMap.objects.all(),
        label=_('Route Map'),
        required=False,
        selector=True
    )

    model = RouteMapEntry
    fieldsets = (
        (None, ('route_map', )),
    )
    nullable_fields = ()



import django_tables2 as tables
from django.utils.translation import gettext_lazy as _

from netbox.tables import NetBoxTable
from netbox_routing.models import StaticRoute


class StaticRouteTable(NetBoxTable):
    devices = tables.ManyToManyColumn(
        verbose_name=_('Devices'),
    )
    class Meta(NetBoxTable.Meta):
        model = StaticRoute
        fields = ('pk', 'id', 'devices', 'vrf', 'prefix', 'next_hop', 'name')
        default_columns = ('pk', 'id', 'devices', 'vrf', 'prefix', 'next_hop', 'name')



from .static import StaticRouteFilterForm
from .objects import PrefixListFilterForm, PrefixListEntryFilterForm, RouteMapFilterForm,\
    RouteMapEntryFilterForm
from .ospf import OSPFAreaFilterForm, OSPFInstanceFilterForm, OSPFInterfaceFilterForm

__all__ = (
    # Static
    'StaticRouteFilterForm',

    # OSPF
    'OSPFAreaFilterForm',
    'OSPFInstanceFilterForm',
    'OSPFInterfaceFilterForm',

    # Routing Objects
    'PrefixListFilterForm',
    'PrefixListEntryFilterForm',
    'RouteMapFilterForm',
    'RouteMapEntryFilterForm'
)



from rest_framework import serializers

from netbox.api.serializers import WritableNestedSerializer
from netbox_routing.models import PrefixList, PrefixListEntry, RouteMap, RouteMapEntry


__all__ = (
    'NestedStaticRouteSerializer'
)


class NestedPrefixListSerializer(WritableNestedSerializer):
    url = serializers.HyperlinkedIdentityField(view_name='plugins-api:netbox_routing-api:prefixlist-detail')

    class Meta:
        model = PrefixList
        fields = ('url', 'id', 'display', 'name')


class NestedPrefixListEntrySerializer(WritableNestedSerializer):
    url = serializers.HyperlinkedIdentityField(view_name='plugins-api:netbox_routing-api:prefixlist-detail')
    prefix_list = NestedPrefixListSerializer

    class Meta:
        model = PrefixListEntry
        fields = ('url', 'id', 'display', 'prefix_list')


class NestedRouteMapSerializer(WritableNestedSerializer):
    url = serializers.HyperlinkedIdentityField(view_name='plugins-api:netbox_routing-api:prefixlist-detail')

    class Meta:
        model = RouteMap
        fields = ('url', 'id', 'display', 'name')


class NestedRouteMapEntrySerializer(WritableNestedSerializer):
    url = serializers.HyperlinkedIdentityField(view_name='plugins-api:netbox_routing-api:prefixlist-detail')
    route_map = NestedRouteMapSerializer

    class Meta:
        model = RouteMapEntry
        fields = ('url', 'id', 'display', 'route_map')



from dcim.filtersets import DeviceFilterSet
from dcim.models import Device
from dcim.tables import DeviceTable
from netbox.views.generic import ObjectListView, ObjectEditView, ObjectView, ObjectDeleteView, ObjectChildrenView
from netbox_routing.filtersets.static import StaticRouteFilterSet
from netbox_routing.forms import StaticRouteForm
from netbox_routing.forms.filtersets.static import StaticRouteFilterForm
from netbox_routing.models import StaticRoute
from netbox_routing.tables.static import StaticRouteTable


__all__ = (
    'StaticRouteListView',
    'StaticRouteView',
    'StaticRouteDevicesView',
    'StaticRouteEditView',
    'StaticRouteDeleteView',
)

from utilities.views import register_model_view, ViewTab


@register_model_view(StaticRoute, name='list')
class StaticRouteListView(ObjectListView):
    queryset = StaticRoute.objects.all()
    table = StaticRouteTable
    filterset = StaticRouteFilterSet
    filterset_form = StaticRouteFilterForm


@register_model_view(StaticRoute)
class StaticRouteView(ObjectView):
    queryset = StaticRoute.objects.all()
    template_name = 'netbox_routing/staticroute.html'


@register_model_view(StaticRoute, name='devices')
class StaticRouteDevicesView(ObjectChildrenView):
    template_name = 'netbox_routing/staticroute_devices.html'
    queryset = StaticRoute.objects.all()
    child_model = Device
    table = DeviceTable
    filterset = DeviceFilterSet
    actions = []
    tab = ViewTab(
        label='Assigned Devices',
        badge=lambda obj: Device.objects.filter(static_routes=obj).count(),
    )

    def get_children(self, request, parent):
        return self.child_model.objects.filter(static_routes=parent)


@register_model_view(StaticRoute, name='edit')
class StaticRouteEditView(ObjectEditView):
    queryset = StaticRoute.objects.all()
    form = StaticRouteForm


@register_model_view(StaticRoute, name='delete')
class StaticRouteDeleteView(ObjectDeleteView):
    pass



from netbox.forms import NetBoxModelFilterSetForm
from netbox_routing.models import PrefixList, PrefixListEntry, RouteMap, RouteMapEntry


class PrefixListFilterForm(NetBoxModelFilterSetForm):
    model = PrefixList


class PrefixListEntryFilterForm(NetBoxModelFilterSetForm):
    model = PrefixListEntry


class RouteMapFilterForm(NetBoxModelFilterSetForm):
    model = RouteMap


class RouteMapEntryFilterForm(NetBoxModelFilterSetForm):
    model = RouteMapEntry


from netbox.api.viewsets import NetBoxModelViewSet
from netbox_routing import filtersets
from netbox_routing.api.serializers import PrefixListSerializer, PrefixListEntrySerializer, RouteMapSerializer, \
    RouteMapEntrySerializer
from netbox_routing.models import PrefixList, PrefixListEntry, RouteMap, RouteMapEntry


class PrefixListViewSet(NetBoxModelViewSet):
    queryset = PrefixList.objects.all()
    serializer_class = PrefixListSerializer
    filterset_class = filtersets.PrefixListFilterSet


class PrefixListEntryViewSet(NetBoxModelViewSet):
    queryset = PrefixListEntry.objects.all()
    serializer_class = PrefixListEntrySerializer
    filterset_class = filtersets.PrefixListEntryFilterSet


class RouteMapViewSet(NetBoxModelViewSet):
    queryset = RouteMap.objects.all()
    serializer_class = RouteMapSerializer
    filterset_class = filtersets.RouteMapFilterSet


class RouteMapEntryViewSet(NetBoxModelViewSet):
    queryset = RouteMapEntry.objects.all()
    serializer_class = RouteMapEntrySerializer
    filterset_class = filtersets.RouteMapEntryFilterSet



import django_tables2 as tables
from django.utils.translation import gettext_lazy as _

from netbox.tables import NetBoxTable
from netbox_routing.models import OSPFArea, OSPFInstance, OSPFInterface


__all__ = (
    'OSPFAreaTable',
    'OSPFInstanceTable',
    'OSPFInterfaceTable',
)


class OSPFInstanceTable(NetBoxTable):
    class Meta(NetBoxTable.Meta):
        model = OSPFInstance
        fields = ('pk', 'id', 'name', 'router_id', 'process_id', 'device')
        default_columns = ('pk', 'id', 'name', 'router_id', 'process_id', 'device')


class OSPFAreaTable(NetBoxTable):
    class Meta(NetBoxTable.Meta):
        model = OSPFArea
        fields = ('pk', 'id', 'area_id')
        default_columns = ('pk', 'id', 'area_id')


class OSPFInterfaceTable(NetBoxTable):
    instance = tables.Column(
        verbose_name=_('Instance'),
        linkify=True
    )
    area = tables.Column(
        verbose_name=_('Area'),
        linkify=True
    )
    interface = tables.Column(
        verbose_name=_('Interface'),
        linkify=True
    )

    class Meta(NetBoxTable.Meta):
        model = OSPFInterface
        fields = ('pk', 'id', 'instance', 'area', 'interface', 'priority', 'bfd', 'authentication', 'passphrase')
        default_columns = ('pk', 'id', 'instance', 'area', 'interface')



from django.utils.translation import gettext as _

from netbox.forms import NetBoxModelBulkEditForm
from netbox_routing.models import OSPFArea, OSPFInstance, OSPFInterface
from utilities.forms.fields import DynamicModelChoiceField


__all__ = (
    'OSPFInterfaceBulkEditForm',
)


class OSPFInterfaceBulkEditForm(NetBoxModelBulkEditForm):
    instance = DynamicModelChoiceField(
        queryset=OSPFInstance.objects.all(),
        label=_('Route Map'),
        required=False,
        selector=True
    )
    area = DynamicModelChoiceField(
        queryset=OSPFArea.objects.all(),
        label=_('Route Map'),
        required=False,
        selector=True
    )

    model = OSPFInterface
    fieldsets = (
        ('OSPF', ('instance', 'area')),
        ('Attributes', ('priority', 'bfd', 'authentication', 'passphrase')),
    )
    nullable_fields = ()



from netbox.forms import NetBoxModelForm
from netbox_routing.models import PrefixList, PrefixListEntry, RouteMap, RouteMapEntry


class PrefixListForm(NetBoxModelForm):

    class Meta:
        model = PrefixList
        fields = ('name',)


class PrefixListEntryForm(NetBoxModelForm):

    class Meta:
        model = PrefixListEntry
        fields = ('prefix_list', 'sequence', 'type', 'prefix', 'le', 'ge')


class RouteMapForm(NetBoxModelForm):

    class Meta:
        model = RouteMap
        fields = ('name',)


class RouteMapEntryForm(NetBoxModelForm):

    class Meta:
        model = RouteMapEntry
        fields = ('route_map', 'sequence', 'type')



from .filtersets import *
from .bulk_edit import *
from .bulk_import import *
from .objects import PrefixListForm, PrefixListEntryForm, RouteMapForm, RouteMapEntryForm
from .ospf import OSPFAreaForm, OSPFInstanceForm, OSPFInterfaceForm
from .static import StaticRouteForm

__all__ = (
    # Static Routes
    'StaticRouteForm',
    'StaticRouteFilterForm',

    # OSPF
    'OSPFAreaForm',
    'OSPFAreaFilterForm',

    'OSPFInstanceForm',
    'OSPFInstanceFilterForm',

    'OSPFInterfaceForm',
    'OSPFInterfaceFilterForm',
    'OSPFInterfaceBulkEditForm',

    # Objects
    'PrefixListForm',
    'PrefixListEntryForm',
    'RouteMapForm',
    'RouteMapEntryForm',
    'PrefixListFilterForm',
    'PrefixListEntryFilterForm',
    'RouteMapFilterForm',
    'RouteMapEntryFilterForm'
)



from django.db import models
from django.urls import reverse
from django.utils.translation import gettext as _

from netbox.models import NetBoxModel

from netbox_routing import choices
from netbox_routing.fields.ip import IPAddressField


__all__ = (
    'OSPFInstance',
    'OSPFArea',
    'OSPFInterface',
)


class OSPFInstance(NetBoxModel):
    name = models.CharField(max_length=100)
    router_id = IPAddressField(verbose_name=_('Router ID'))
    process_id = models.IntegerField(verbose_name=_('Process ID'))
    device = models.ForeignKey(
        to='dcim.Device',
        related_name='ospf_instances',
        on_delete=models.CASCADE,
        blank=False,
        null=False
    )

    clone_fields = ('device',)
    prerequisite_models = (
        'dcim.Device',
    )

    class Meta:
        verbose_name = 'OSPF Instance'

    def __str__(self):
        return f'{self.name} ({self.router_id})'

    def get_absolute_url(self):
        return reverse('plugins:netbox_routing:ospfinstance', args=[self.pk])


class OSPFArea(NetBoxModel):
    area_id = models.CharField(max_length=100, verbose_name='Area ID')

    prerequisite_models = ()
    clone_fields = ()
    class Meta:
        verbose_name = 'OSPF Area'

    def __str__(self):
        return f'{self.area_id}'

    def get_absolute_url(self):
        return reverse('plugins:netbox_routing:ospfarea', args=[self.pk])


class OSPFInterface(NetBoxModel):
    instance = models.ForeignKey(
        to='netbox_routing.OSPFInstance',
        related_name='interfaces',
        on_delete=models.PROTECT,
        blank=False,
        null=False,
    )
    area = models.ForeignKey(
        to='netbox_routing.OSPFArea',
        on_delete=models.CASCADE,
        related_name='interfaces',
        blank=False,
        null=False
    )
    interface = models.ForeignKey(
        to='dcim.Interface',
        related_name='ospf_interfaces',
        on_delete=models.CASCADE,
        blank=False,
        null=False
    )
    priority = models.IntegerField(blank=True, null=True)
    bfd = models.BooleanField(blank=True, null=True, verbose_name='BFD')
    authentication = models.CharField(
        max_length=50,
        choices=choices.AuthenticationChoices,
        blank=True,
        null=True
    )
    passphrase = models.CharField(
        max_length=200,
        blank=True,
        null=True
    )

    clone_fields = ('instance', 'area', 'priority', 'bfd', 'authentication', 'passphrase')
    prerequisite_models = (
        'netbox_routing.OSPFInstance', 'netbox_routing.OSPFArea', 'dcim.Interface',
    )

    class Meta:
        verbose_name = 'OSPF Interface'
        ordering = ('instance', 'area', 'interface')  # Name may be non-unique
        constraints = (
            models.UniqueConstraint(
                fields=('interface', ),
                name='%(app_label)s_%(class)s_unique_interface'
            ),
        )

    def __str__(self):
        return f'{self.interface.name}'

    def get_absolute_url(self):
        return reverse('plugins:netbox_routing:ospfinterface', args=[self.pk])




import django_filters
import netaddr
from django.db.models import Q

from netbox.filtersets import NetBoxModelFilterSet
from netbox_routing.models import PrefixList, PrefixListEntry, RouteMapEntry, RouteMap


class PrefixListFilterSet(NetBoxModelFilterSet):
    class Meta:
        model = PrefixList
        fields = ()

    def search(self, queryset, name, value):
        if not value.strip():
            return queryset
        qs_filter = (
            Q(name=value)
        )
        return queryset.filter(qs_filter).distinct()


class PrefixListEntryFilterSet(NetBoxModelFilterSet):

    prefix = django_filters.CharFilter(
        method='filter_prefix',
        label='Prefix',
    )

    class Meta:
        model = PrefixListEntry
        fields = ('prefix_list', 'prefix', 'sequence', 'type', 'le', 'ge')

    def search(self, queryset, name, value):
        if not value.strip():
            return queryset
        qs_filter = (
            Q(prefix_list__name__icontains=value) |
            Q(prefix__icontains=value) |
            Q(type=value)
        )
        return queryset.filter(qs_filter).distinct()

    def filter_prefix(self, queryset, name, value):
        if not value.strip():
            return queryset
        try:
            query = str(netaddr.IPNetwork(value).cidr)
            return queryset.filter(prefix=query)
        except (netaddr.AddrFormatError, ValueError):
            return queryset.none()


class RouteMapFilterSet(NetBoxModelFilterSet):

    class Meta:
        model = RouteMap
        fields = ()

    def search(self, queryset, name, value):
        if not value.strip():
            return queryset
        qs_filter = (
            Q(name__icontains=value)
        )
        return queryset.filter(qs_filter).distinct()


class RouteMapEntryFilterSet(NetBoxModelFilterSet):

    class Meta:
        model = RouteMapEntry
        fields = ('route_map', 'sequence', 'type')

    def search(self, queryset, name, value):
        if not value.strip():
            return queryset
        qs_filter = (
            Q(route_map__name__icontains=value) |
            Q(type=value)
        )
        return queryset.filter(qs_filter).distinct()



from .static import StaticRouteListView, StaticRouteDevicesView, StaticRouteEditView, StaticRouteView, \
    StaticRouteDeleteView

from .objects import PrefixListView, PrefixListEditView, PrefixListListView, PrefixListDeleteView, RouteMapListView, \
    RouteMapView, RouteMapEditView, RouteMapDeleteView, PrefixListEntryListView, PrefixListEntryEditView, \
    PrefixListEntryDeleteView, PrefixListEntryView, RouteMapEntryListView, RouteMapEntryView, RouteMapEntryEditView, \
    RouteMapEntryDeleteView, PrefixListEntriesView, RouteMapEntriesView, RouteMapEntryBulkEditView, \
    RouteMapEntryBulkDeleteView, PrefixListEntryBulkDeleteView, PrefixListEntryBulkEditView

from .ospf import *
from .core import *

__all__ = (
    # Core View Extensions
    'DeviceStaticRoutesView',

    # Static
    'StaticRouteListView',
    'StaticRouteView',
    'StaticRouteDevicesView',
    'StaticRouteEditView',
    'StaticRouteDeleteView',

    # OSPF
    'OSPFInstanceListView',
    'OSPFInstanceView',
    'OSPFInstanceEditView',
    'OSPFInstanceDeleteView',
    'OSPFInstanceInterfacesView',

    'OSPFAreaListView',
    'OSPFAreaView',
    'OSPFAreaInterfacesView',
    'OSPFAreaEditView',
    'OSPFAreaDeleteView',

    'OSPFInterfaceListView',
    'OSPFInterfaceView',
    'OSPFInterfaceEditView',
    'OSPFInterfaceDeleteView',


    # Routing Objects
    'PrefixListListView',
    'PrefixListView',
    'PrefixListEntriesView',
    'PrefixListEditView',
    'PrefixListDeleteView',
    'PrefixListEntryListView',
    'PrefixListEntryView',
    'PrefixListEntryEditView',
    'PrefixListEntryDeleteView',
    'PrefixListEntryBulkEditView',
    'PrefixListEntryBulkDeleteView',

    'RouteMapListView',
    'RouteMapView',
    'RouteMapEntriesView',
    'RouteMapEditView',
    'RouteMapDeleteView',
    'RouteMapEntryListView',
    'RouteMapEntryView',
    'RouteMapEntryEditView',
    'RouteMapEntryDeleteView',
    'RouteMapEntryBulkEditView',
    'RouteMapEntryBulkDeleteView',

)



import django_filters
from django.core.exceptions import ValidationError
from django.db.models import Q
from django.utils.translation import gettext as _

from dcim.models import Device
from utilities.filters import MultiValueCharFilter

from netbox.filtersets import NetBoxModelFilterSet
from netbox_routing.models import OSPFArea, OSPFInstance, OSPFInterface


__all__ = (
    'OSPFAreaFilterSet',
    'OSPFInstanceFilterSet',
    'OSPFInterfaceFilterSet'
)



class OSPFInstanceFilterSet(NetBoxModelFilterSet):
    device_id = django_filters.ModelMultipleChoiceFilter(
        field_name='device',
        queryset=Device.objects.all(),
        label='Device (ID)',
    )
    device = django_filters.ModelMultipleChoiceFilter(
        field_name='device__name',
        queryset=Device.objects.all(),
        to_field_name='name',
        label='Device',
    )

    router_id = MultiValueCharFilter(
        method='filter_rid',
        label=_('Router ID'),
    )

    class Meta:
        model = OSPFInstance
        fields = ('device_id', 'device', 'name', 'device', 'router_id', 'process_id')

    def search(self, queryset, name, value):
        if not value.strip():
            return queryset
        qs_filter = (
            Q(name__icontains=value) |
            Q(device__name__icontains=value) |
            Q(router_id__icontains=value)
        )
        return queryset.filter(qs_filter).distinct()

    def filter_rid(self, queryset, name, value):
        try:
            return queryset.filter(**{f'{name}__in': value})
        except ValidationError:
            return queryset.none()


class OSPFAreaFilterSet(NetBoxModelFilterSet):
    area_id = MultiValueCharFilter(
        method='filter_aid',
        label=_('Area ID'),
    )

    class Meta:
        model = OSPFArea
        fields = ('area_id', )

    def search(self, queryset, name, value):
        if not value.strip():
            return queryset
        qs_filter = (
            Q(area_id__icontains=value)
        )
        return queryset.filter(qs_filter).distinct()

    def filter_aid(self, queryset, name, value):
        try:
            return queryset.filter(**{f'{name}__in': value})
        except ValidationError:
            return queryset.none()


class OSPFInterfaceFilterSet(NetBoxModelFilterSet):

    class Meta:
        model = OSPFInterface
        fields = ('instance', 'area', 'interface', 'bfd', 'priority', 'authentication', 'passphrase')

    def search(self, queryset, name, value):
        if not value.strip():
            return queryset
        qs_filter = (
            Q(instance__name__icontains=value) |
            Q(area__area_id__icontains=value) |
            Q(interface__name__icontains=value) |
            Q(interface__label__icontains=value) |
            Q(interface__device__name__icontains=value)
        )
        return queryset.filter(qs_filter).distinct()




from dcim.models import Device
from ipam.models import VRF
from netbox.forms import NetBoxModelForm
from netbox_routing.models import StaticRoute
from utilities.forms.fields import DynamicModelChoiceField, DynamicModelMultipleChoiceField


class StaticRouteForm(NetBoxModelForm):
    devices = DynamicModelMultipleChoiceField(
        queryset=Device.objects.all()
    )
    vrf = DynamicModelChoiceField(
        queryset=VRF.objects.all(),
        required=False,
        label='VRF'
    )

    class Meta:
        model = StaticRoute
        fields = ('devices', 'vrf', 'prefix', 'next_hop', 'name', 'metric', 'permanent')

    def __init__(self, data=None, instance=None, *args, **kwargs):
        super().__init__(data=data, instance=instance, *args, **kwargs)

        if self.instance and self.instance.pk is not None:
            self.fields['devices'].initial = self.instance.devices.all().values_list('id', flat=True)

    def save(self, *args, **kwargs):
        instance = super().save(*args, **kwargs)
        instance.devices.set(self.cleaned_data['devices'])
        return instance



from rest_framework import serializers

from netbox.api.serializers import WritableNestedSerializer
from netbox_routing.models import StaticRoute


__all__ = (
    'NestedStaticRouteSerializer'
)


class NestedStaticRouteSerializer(WritableNestedSerializer):
    url = serializers.HyperlinkedIdentityField(view_name='plugins-api:netbox_routing-api:staticroute-detail')

    class Meta:
        model = StaticRoute
        fields = ('url', 'id', 'display', 'prefix', 'next_hop', 'name', 'metric', 'permanent')


from .static import StaticRouteFilterSet
from .objects import PrefixListFilterSet, PrefixListEntryFilterSet, RouteMapFilterSet, RouteMapEntryFilterSet
from .ospf import *


__all__ = (
    'StaticRouteFilterSet',

    'PrefixListFilterSet',
    'PrefixListEntryFilterSet',
    'RouteMapFilterSet',
    'RouteMapEntryFilterSet',

    'OSPFInstanceFilterSet',
    'OSPFAreaFilterSet',
    'OSPFInterfaceFilterSet',
)



from .static import StaticRouteViewSet
from .ospf import OSPFInstanceViewSet, OSPFAreaViewSet, OSPFInterfaceViewSet
from .objects import PrefixListViewSet, PrefixListEntryViewSet, RouteMapViewSet, RouteMapEntryViewSet

__all__ = (
    'StaticRouteViewSet',
    'OSPFInstanceViewSet',
    'OSPFAreaViewSet',
    'OSPFInterfaceViewSet',
    'PrefixListViewSet',
    'PrefixListEntryViewSet',
    'RouteMapViewSet',
    'RouteMapEntryViewSet',
)



from rest_framework import serializers

from dcim.api.nested_serializers import NestedInterfaceSerializer, NestedDeviceSerializer
from netbox.api.serializers import NetBoxModelSerializer
from netbox_routing.api.nested_serializers.ospf import NestedOSPFInstanceSerializer, NestedOSPFAreaSerializer
from netbox_routing.models import OSPFInstance, OSPFArea, OSPFInterface


__all__ = (
    'OSPFInstanceSerializer',
    'OSPFAreaSerializer',
    'OSPFInterfaceSerializer',
)


class OSPFInstanceSerializer(NetBoxModelSerializer):
    url = serializers.HyperlinkedIdentityField(view_name='plugins-api:netbox_routing-api:ospfinstance-detail')
    device = NestedDeviceSerializer()

    class Meta:
        model = OSPFInstance
        fields = ('url', 'id', 'display', 'name', 'router_id', 'process_id', 'device')


class OSPFAreaSerializer(NetBoxModelSerializer):
    url = serializers.HyperlinkedIdentityField(view_name='plugins-api:netbox_routing-api:ospfarea-detail')

    class Meta:
        model = OSPFArea
        fields = ('url', 'id', 'display', 'area_id')


class OSPFInterfaceSerializer(NetBoxModelSerializer):
    url = serializers.HyperlinkedIdentityField(view_name='plugins-api:netbox_routing-api:ospfarea-detail')
    instance = NestedOSPFInstanceSerializer()
    area = NestedOSPFAreaSerializer()
    interface = NestedInterfaceSerializer()

    class Meta:
        model = OSPFInterface
        fields = (
            'url', 'id', 'display', 'instance', 'area', 'interface', 'priority', 'bfd', 'authentication', 'passphrase'
        )



from django.contrib.contenttypes.fields import GenericForeignKey
from django.contrib.contenttypes.models import ContentType
from django.db import models

from netbox.models import NetBoxModel, NestedGroupModel
from netbox_routing import choices
from netbox_routing.constants.bgp import BGPSETTING_ASSIGNMENT_MODELS, BGPAF_ASSIGNMENT_MODELS, \
    BGPPEER_ASSIGNMENT_MODELS, BGPPEERAF_ASSIGNMENT_MODELS
from netbox_routing.fields.ip import IPAddressField


class BGPSettings(NetBoxModel):
    assigned_object_type = models.ForeignKey(
        to=ContentType,
        limit_choices_to=BGPSETTING_ASSIGNMENT_MODELS,
        on_delete=models.PROTECT,
        related_name='+',
        blank=True,
        null=True
    )
    assigned_object_id = models.PositiveBigIntegerField(
        blank=True,
        null=True
    )
    assigned_object = GenericForeignKey(
        ct_field='assigned_object_type',
        fk_field='assigned_object_id'
    )
    router_id = IPAddressField()
    auto_summary = models.BooleanField()
    bgp_additional_paths_install = models.BooleanField()
    bgp_additional_paths_receive = models.BooleanField()
    bgp_additional_paths_send = models.BooleanField()
    bgp_asnotation_dot = models.BooleanField()
    bgp_graceful_restart = models.BooleanField()

    default_information_originate = models.BooleanField(verbose_name='Default Information Originate')
    default_metric = models.PositiveBigIntegerField(verbose_name='Default Metric')
    distance_ebgp = models.PositiveSmallIntegerField(verbose_name='eBGP Distance')
    distance_ibgp = models.PositiveSmallIntegerField(verbose_name='iBGP Distance')
    distance_embgp = models.PositiveSmallIntegerField(verbose_name='eBGP Distance (MultiProtocol)')
    distance_imbgp = models.PositiveSmallIntegerField(verbose_name='iBGP Distance (MultiProtocol)')
    paths_maximum = models.PositiveSmallIntegerField(verbose_name='Maximum Paths')
    paths_maximum_secondary = models.PositiveSmallIntegerField(verbose_name='Maximum Secondary Paths')
    timers_keepalive = models.PositiveSmallIntegerField(verbose_name='Keepalive Timer')
    timers_hold = models.PositiveSmallIntegerField(verbose_name='Hold Timer')


class BGPRouter(NetBoxModel):
    asn = models.ForeignKey(
        to='ipam.ASN',
        on_delete=models.PROTECT,
        related_name='router',
        verbose_name='ASN'
    )


class BGPScope(NetBoxModel):
    router = models.ForeignKey(
        to=BGPRouter,
        on_delete=models.PROTECT,
        related_name='scopes',
        blank=False,
        null=False
    )
    vrf = models.ForeignKey(
        to='ipam.VRF',
        on_delete=models.PROTECT,
        related_name='scopes',
        blank=False,
        null=False
    )


class BGPAddressFamily(NetBoxModel):
    assigned_object_type = models.ForeignKey(
        to=ContentType,
        limit_choices_to=BGPAF_ASSIGNMENT_MODELS,
        on_delete=models.PROTECT,
        related_name='+',
        blank=True,
        null=True
    )
    assigned_object_id = models.PositiveBigIntegerField(
        blank=True,
        null=True
    )
    assigned_object = GenericForeignKey(
        ct_field='assigned_object_type',
        fk_field='assigned_object_id'
    )


class BGPTemplateSession(NetBoxModel):
    name = models.CharField(
        verbose_name='Name',
        max_length=255
    )
    router = models.ForeignKey(
        to=BGPRouter,
        on_delete=models.PROTECT,
        related_name='session_templates',
        blank=False,
        null=False
    )
    enabled = models.BooleanField(
        blank=True,
        null=True
    )
    prefixlist_out = models.ForeignKey(
        to='netbox_routing.PrefixList',
        on_delete=models.PROTECT,
        related_name='template_afs_out',
        blank=True,
        null=True
    )
    prefixlist_in = models.ForeignKey(
        to='netbox_routing.PrefixList',
        on_delete=models.PROTECT,
        related_name='template_afs_in',
        blank=True,
        null=True
    )
    routemap_out = models.ForeignKey(
        to='netbox_routing.RouteMap',
        on_delete=models.PROTECT,
        related_name='template_afs_out',
        blank=True,
        null=True
    )
    routemap_in = models.ForeignKey(
        to='netbox_routing.RouteMap',
        on_delete=models.PROTECT,
        related_name='template_afs_in',
        blank=True,
        null=True
    )


class BGPTemplatePolicy(NetBoxModel):
    name = models.CharField(
        verbose_name='Name',
        max_length=255
    )
    router = models.ForeignKey(
        to=BGPRouter,
        on_delete=models.PROTECT,
        related_name='session_templates',
        blank=False,
        null=False
    )
    enabled = models.BooleanField(
        blank=True,
        null=True
    )


class BGPTemplatePeer(NetBoxModel):
    name = models.CharField(
        verbose_name='Name',
        max_length=255
    )
    remote_as = models.ForeignKey(
        to='ipam.ASN',
        on_delete=models.PROTECT,
        related_name='+',
        blank=True,
        null=True
    )
    enabled = models.BooleanField(
        blank=True,
        null=True
    )


class BGPPeerGroup(NetBoxModel):
    name = models.CharField(
        verbose_name='Name',
        max_length=255
    )
    remote_as = models.ForeignKey(
        to='ipam.ASN',
        on_delete=models.PROTECT,
        related_name='+',
        blank=True,
        null=True
    )
    enabled = models.BooleanField(
        blank=True,
        null=True
    )


class BGPPeer(NetBoxModel):
    assigned_object_type = models.ForeignKey(
        to=ContentType,
        limit_choices_to=BGPPEER_ASSIGNMENT_MODELS,
        on_delete=models.PROTECT,
        related_name='+',
        blank=True,
        null=True
    )
    assigned_object_id = models.PositiveBigIntegerField(
        blank=True,
        null=True
    )
    assigned_object = GenericForeignKey(
        ct_field='assigned_object_type',
        fk_field='assigned_object_id'
    )
    peer = IPAddressField()
    peer_group = models.ForeignKey(
        to=BGPPeerGroup,
        on_delete=models.PROTECT,
        related_name='peers',
        blank=True,
        null=True
    )
    peer_template = models.ForeignKey(
        to=BGPTemplatePeer,
        on_delete=models.PROTECT,
        related_name='peers',
        blank=True,
        null=True
    )
    peer_policy = models.ForeignKey(
        to=BGPTemplatePolicy,
        on_delete=models.PROTECT,
        related_name='peers',
        blank=True,
        null=True
    )
    remote_as = models.ForeignKey(
        to='ipam.ASN',
        on_delete=models.PROTECT,
        related_name='+',
        blank=True,
        null=True
    )
    enabled = models.BooleanField(
        blank=True,
        null=True
    )


class BGPPeerAddressFamily(NetBoxModel):
    assigned_object_type = models.ForeignKey(
        to=ContentType,
        limit_choices_to=BGPPEERAF_ASSIGNMENT_MODELS,
        on_delete=models.PROTECT,
        related_name='+',
        blank=True,
        null=True
    )
    assigned_object_id = models.PositiveBigIntegerField(
        blank=True,
        null=True
    )
    assigned_object = GenericForeignKey(
        ct_field='assigned_object_type',
        fk_field='assigned_object_id'
    )

    peer_session = models.ForeignKey(
        to=BGPTemplateSession,
        on_delete=models.PROTECT,
        related_name='peer_afs',
        blank=True,
        null=True
    )

    address_family = models.CharField(
        choices=choices.BGPAddressFamilies
    )
    enabled = models.BooleanField(
        blank=True,
        null=True
    )

    prefixlist_out = models.ForeignKey(
        to='netbox_routing.PrefixList',
        on_delete=models.PROTECT,
        related_name='peer_afs_out',
        blank=True,
        null=True
    )
    prefixlist_in = models.ForeignKey(
        to='netbox_routing.PrefixList',
        on_delete=models.PROTECT,
        related_name='peer_afs_in',
        blank=True,
        null=True
    )
    routemap_out = models.ForeignKey(
        to='netbox_routing.RouteMap',
        on_delete=models.PROTECT,
        related_name='peer_afs_out',
        blank=True,
        null=True
    )
    routemap_in = models.ForeignKey(
        to='netbox_routing.RouteMap',
        on_delete=models.PROTECT,
        related_name='peer_afs_in',
        blank=True,
        null=True
    )



from extras.plugins import PluginMenuButton, PluginMenuItem, PluginMenu
from utilities.choices import ButtonColorChoices

static = PluginMenuItem(
    link='plugins:netbox_routing:staticroute_list',
    link_text='Static Route',
    permissions=['netbox_routing.view_staticroute'],
    buttons=(
        PluginMenuButton('plugins:netbox_routing:staticroute_add', 'Add', 'mdi mdi-plus', ButtonColorChoices.GREEN),
        PluginMenuButton(
            'plugins:netbox_routing:staticroute_import',
            'Import',
            'mdi mdi-upload',
            ButtonColorChoices.CYAN
        ),
    )
)

ospf_instance = PluginMenuItem(
    link='plugins:netbox_routing:ospfinstance_list',
    link_text='Instances',
    permissions=['netbox_routing.view_ospfinstance'],
    buttons=(
        PluginMenuButton('plugins:netbox_routing:ospfinstance_add', 'Add', 'mdi mdi-plus', ButtonColorChoices.GREEN),
        PluginMenuButton(
            'plugins:netbox_routing:ospfinstance_import',
            'Import',
            'mdi mdi-upload',
            ButtonColorChoices.CYAN
        ),
    )
)
ospf_area = PluginMenuItem(
    link='plugins:netbox_routing:ospfarea_list',
    link_text='Areas',
    permissions=['netbox_routing.view_ospfarea'],
    buttons=(
        PluginMenuButton('plugins:netbox_routing:ospfarea_add', 'Add', 'mdi mdi-plus', ButtonColorChoices.GREEN),
        PluginMenuButton(
            'plugins:netbox_routing:ospfarea_import',
            'Import',
            'mdi mdi-upload',
            ButtonColorChoices.CYAN
        ),
    )
)
ospf_interfaces = PluginMenuItem(
    link='plugins:netbox_routing:ospfinterface_list',
    link_text='Interfaces',
    permissions=['netbox_routing.view_ospfinterface'],
    buttons=(
        PluginMenuButton('plugins:netbox_routing:ospfinterface_add', 'Add', 'mdi mdi-plus', ButtonColorChoices.GREEN),
        PluginMenuButton(
            'plugins:netbox_routing:ospfinterface_import',
            'Import',
            'mdi mdi-upload',
            ButtonColorChoices.CYAN
        ),
    )
)

prefixlist = PluginMenuItem(
    link='plugins:netbox_routing:prefixlist_list',
    link_text='Prefix Lists',
    permissions=['netbox_routing.view_prefixlist'],
    buttons=(
        PluginMenuButton('plugins:netbox_routing:prefixlist_add', 'Add', 'mdi mdi-plus', ButtonColorChoices.GREEN),
        PluginMenuButton('plugins:netbox_routing:prefixlist_import', 'Import', 'mdi mdi-upload', ButtonColorChoices.CYAN),
    )
)
routemap = PluginMenuItem(
    link='plugins:netbox_routing:routemap_list',
    link_text='Route Maps',
    permissions=['netbox_routing.view_routemap'],
    buttons=(
        PluginMenuButton('plugins:netbox_routing:routemap_add', 'Add', 'mdi mdi-plus', ButtonColorChoices.GREEN),
        PluginMenuButton('plugins:netbox_routing:routemap_import', 'Import', 'mdi mdi-upload', ButtonColorChoices.CYAN),
    )
)

menu = PluginMenu(
    label='Netbox Routing',
    groups=(
        ('Routing Objects', (prefixlist, routemap)),
        ('Static Routing', (static,)),
        ('OSPF', (ospf_instance, ospf_area, ospf_interfaces)),
    ),
    icon_class='mdi mdi-router'
)



from .static import StaticRouteSerializer
from .ospf import *
from .objects import PrefixListSerializer, PrefixListEntrySerializer, RouteMapSerializer, RouteMapEntrySerializer
from ..nested_serializers import *

__all__ = (
    'StaticRouteSerializer',

    'OSPFInstanceSerializer',
    'OSPFAreaSerializer',
    'OSPFInterfaceSerializer',

    'PrefixListSerializer',
    'PrefixListEntrySerializer',
    'RouteMapSerializer',
    'RouteMapEntrySerializer',

    # Nested Serializers
    'NestedPrefixListSerializer',
    'NestedPrefixListEntrySerializer',
    'NestedRouteMapSerializer',
    'NestedRouteMapEntrySerializer',
    'NestedStaticRouteSerializer'

)



from django import forms
from django.utils.translation import gettext as _

from dcim.models import Interface, Device
from netbox.forms import NetBoxModelForm
from utilities.forms import BOOLEAN_WITH_BLANK_CHOICES
from utilities.forms.fields import DynamicModelChoiceField, DynamicModelMultipleChoiceField

from netbox_routing.models import OSPFArea, OSPFInstance, OSPFInterface


__all__ = (
    'OSPFAreaForm',
    'OSPFInstanceForm',
    'OSPFInterfaceForm',
)


class OSPFInstanceForm(NetBoxModelForm):
    device = DynamicModelChoiceField(
        queryset=Device.objects.all(),
        required=True,
        selector=True,
        label=_('Device'),
    )

    class Meta:
        model = OSPFInstance
        fields = ('name', 'router_id', 'process_id', 'device')


class OSPFAreaForm(NetBoxModelForm):

    class Meta:
        model = OSPFArea
        fields = ('area_id', )


class OSPFInterfaceForm(NetBoxModelForm):
    device = DynamicModelChoiceField(
        queryset=Device.objects.all(),
        required=True,
        selector=True,
        label=_('Device'),
    )
    instance = DynamicModelChoiceField(
        queryset=OSPFInstance.objects.all(),
        required=True,
        selector=True,
        label=_('Instance'),
        query_params={
            'device_id': '$device',
        }
    )
    area = DynamicModelChoiceField(
        queryset=OSPFArea.objects.all(),
        required=True,
        selector=True,
        label=_('Area'),
    )
    interface = DynamicModelChoiceField(
        queryset=Interface.objects.all(),
        required=True,
        selector=True,
        label=_('Interface'),
        query_params={
            'device_id': '$device',
        }
    )


    class Meta:
        model = OSPFInterface
        fields = ('device', 'instance', 'area', 'interface', 'priority', 'bfd', 'authentication', 'passphrase')

        widgets = {
            'bfd': forms.Select(choices=BOOLEAN_WITH_BLANK_CHOICES),
        }

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        if self.instance.pk:
            self.initial['device'] = self.instance.interface.device.pk



from .objects import *
from .ospf import OSPFInterfaceBulkEditForm


__all__ = (

    # OSPF
    'OSPFInterfaceBulkEditForm',

    # Route Objects
    'PrefixListEntryBulkEditForm',
    'RouteMapEntryBulkEditForm'
)




from utilities.choices import ChoiceSet


class PermitDenyChoices(ChoiceSet):
    PERMIT = 'permit'
    DENY = 'deny'

    CHOICES = [
        (PERMIT, 'Permit', 'blue'),
        (DENY, 'Deny', 'red')
    ]



# Generated by Django 4.1.10 on 2023-08-09 19:07

from django.db import migrations, models
import django.db.models.deletion
import netbox_routing.fields.ip
import taggit.managers
import utilities.json


class Migration(migrations.Migration):

    dependencies = [
        ('dcim', '0172_larger_power_draw_values'),
        ('extras', '0092_delete_jobresult'),
        ('netbox_routing', '0004_alter_prefixlistentry_ge_alter_prefixlistentry_le'),
    ]

    operations = [
        migrations.CreateModel(
            name='OSPFArea',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False)),
                ('created', models.DateTimeField(auto_now_add=True, null=True)),
                ('last_updated', models.DateTimeField(auto_now=True, null=True)),
                ('custom_field_data', models.JSONField(blank=True, default=dict, encoder=utilities.json.CustomFieldJSONEncoder)),
                ('area_id', models.CharField(max_length=100)),
                ('tags', taggit.managers.TaggableManager(through='extras.TaggedItem', to='extras.Tag')),
            ],
            options={
                'verbose_name': 'OSPF Area',
            },
        ),
        migrations.CreateModel(
            name='OSPFInstance',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False)),
                ('created', models.DateTimeField(auto_now_add=True, null=True)),
                ('last_updated', models.DateTimeField(auto_now=True, null=True)),
                ('custom_field_data', models.JSONField(blank=True, default=dict, encoder=utilities.json.CustomFieldJSONEncoder)),
                ('name', models.CharField(max_length=100)),
                ('router_id', netbox_routing.fields.ip.IPAddressField()),
                ('process_id', models.IntegerField()),
                ('device', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='ospf_instances', to='dcim.device')),
                ('tags', taggit.managers.TaggableManager(through='extras.TaggedItem', to='extras.Tag')),
            ],
            options={
                'verbose_name': 'OSPF Instance',
            },
        ),
        migrations.CreateModel(
            name='OSPFInterface',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False)),
                ('created', models.DateTimeField(auto_now_add=True, null=True)),
                ('last_updated', models.DateTimeField(auto_now=True, null=True)),
                ('custom_field_data', models.JSONField(blank=True, default=dict, encoder=utilities.json.CustomFieldJSONEncoder)),
                ('priority', models.IntegerField(blank=True, null=True)),
                ('bfd', models.BooleanField(blank=True, null=True)),
                ('authentication', models.CharField(blank=True, max_length=50, null=True)),
                ('passphrase', models.CharField(blank=True, max_length=200, null=True)),
                ('area', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='interfaces', to='netbox_routing.ospfarea')),
                ('instance', models.ForeignKey(on_delete=django.db.models.deletion.PROTECT, related_name='interfaces', to='netbox_routing.ospfinstance')),
                ('interface', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='ospf_interfaces', to='dcim.interface')),
                ('tags', taggit.managers.TaggableManager(through='extras.TaggedItem', to='extras.Tag')),
            ],
            options={
                'verbose_name': 'OSPF Interface',
                'ordering': ('instance', 'area', 'interface'),
            },
        ),
        migrations.AddConstraint(
            model_name='ospfinterface',
            constraint=models.UniqueConstraint(fields=('interface',), name='netbox_routing_ospfinterface_unique_interface'),
        ),
    ]



import django_tables2 as tables
from django.utils.translation import gettext_lazy as _

from netbox.tables import NetBoxTable, columns
from netbox_routing.models import PrefixList, PrefixListEntry, RouteMap, RouteMapEntry


class PrefixListTable(NetBoxTable):
    class Meta(NetBoxTable.Meta):
        model = PrefixList
        fields = ('pk', 'id', 'name')
        default_columns = ('pk', 'id', 'name')


class PrefixListEntryTable(NetBoxTable):
    prefix_list = tables.Column(
        verbose_name=_('Prefix List'),
        linkify=True
    )
    type = columns.ChoiceFieldColumn()
    class Meta(NetBoxTable.Meta):
        model = PrefixListEntry
        fields = ('pk', 'id', 'prefix_list', 'sequence', 'type', 'prefix', 'le', 'ge')
        default_columns = ('pk', 'id', 'prefix_list', 'sequence', 'type', 'prefix', 'le', 'ge')


class RouteMapTable(NetBoxTable):
    class Meta(NetBoxTable.Meta):
        model = RouteMap
        fields = ('pk', 'id', 'name')
        default_columns = ('pk', 'id', 'name')


class RouteMapEntryTable(NetBoxTable):
    route_map = tables.Column(
        verbose_name=_('Route Map'),
        linkify=True
    )
    type = columns.ChoiceFieldColumn()
    class Meta(NetBoxTable.Meta):
        model = RouteMapEntry
        fields = ('pk', 'id', 'route_map', 'sequence', 'type')
        default_columns = ('pk', 'id', 'route_map', 'sequence', 'type')
